"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateAddressSpace = exports.readNodeSet2XmlFile = void 0;
const fs_1 = __importDefault(require("fs"));
const util_1 = require("util");
const node_opcua_debug_1 = require("node-opcua-debug");
const __1 = require("..");
const doDebug = (0, node_opcua_debug_1.checkDebugFlag)(__filename);
const debugLog = (0, node_opcua_debug_1.make_debugLog)(__filename);
const errorLog = (0, node_opcua_debug_1.make_errorLog)(__filename);
async function readNodeSet2XmlFile(xmlFile) {
    // istanbul ignore next
    if (!fs_1.default.existsSync(xmlFile)) {
        const msg = "[NODE-OPCUA-E02] generateAddressSpace : cannot find nodeset2 xml file at " + xmlFile;
        errorLog(msg);
        throw new Error(msg);
    }
    debugLog(" parsing ", xmlFile);
    const xmlData = await fs_1.default.promises.readFile(xmlFile, "utf-8");
    return xmlData;
}
exports.readNodeSet2XmlFile = readNodeSet2XmlFile;
function generateAddressSpace(...args) {
    const addressSpace = args[0];
    const xmlFiles = args[1];
    if (args.length === 4) {
        const options = args[2];
        const callback = args[3];
        (0, util_1.callbackify)(__1.generateAddressSpaceRaw)(addressSpace, xmlFiles, readNodeSet2XmlFile, options || {}, callback);
    }
    else {
        const options = {};
        const callback = args[2];
        (0, util_1.callbackify)(__1.generateAddressSpaceRaw)(addressSpace, xmlFiles, readNodeSet2XmlFile, options, callback);
    }
}
exports.generateAddressSpace = generateAddressSpace;
// tslint:disable:no-var-requires
// tslint:disable:max-line-length
const thenify = require("thenify");
module.exports.generateAddressSpace = thenify.withCallback(module.exports.generateAddressSpace);
//# sourceMappingURL=generate_address_space.js.map