import { BinaryStream, OutputBinaryStream } from "node-opcua-binary-stream";
import { Func1, IStructuredTypeSchema, IBaseUAObject, DecodeDebugOptions } from "./types";
export interface BaseUAObject extends IBaseUAObject {
    schema: IStructuredTypeSchema;
}
/**
 * @class BaseUAObject
 * @constructor
 */
export declare class BaseUAObject {
    constructor();
    /**
     * Encode the object to the binary stream.
     * @class BaseUAObject
     * @method encode
     * @param stream {BinaryStream}
     */
    encode(stream: OutputBinaryStream): void;
    /**
     * Decode the object from the binary stream.
     * @class BaseUAObject
     * @method decode
     * @param stream {BinaryStream}
     */
    decode(stream: BinaryStream): void;
    /**
     * Calculate the required size to store this object in a binary stream.
     * @method binaryStoreSize
     * @return number
     */
    binaryStoreSize(): number;
    /**
     * @method toString
     * @return {String}
     */
    toString(...args: any[]): string;
    /**
     *
     * verify that all object attributes values are valid according to schema
     * @method isValid
     * @return boolean
     */
    isValid(): boolean;
    /**
     * @method decodeDebug
     *
     */
    decodeDebug(stream: BinaryStream, options: DecodeDebugOptions): void;
    explore(): string;
    applyOnAllFields<T>(func: Func1<T>, data: T): void;
    toJSON(): any;
    clone(): any;
}
