/**
 * @module node-opcua-secure-channel
 */
/// <reference types="node" />
import { DerivedKeys } from "node-opcua-crypto";
import { BaseUAObject } from "node-opcua-factory";
import { AsymmetricAlgorithmSecurityHeader, SymmetricAlgorithmSecurityHeader } from "node-opcua-service-secure-channel";
import { SecureMessageChunkManagerOptions, SecurityHeader } from "./secure_message_chunk_manager";
export interface MessageChunkerOptions {
    securityHeader?: SecurityHeader;
    derivedKeys?: DerivedKeys | null;
    maxMessageSize?: number;
    maxChunkCount?: number;
}
export type MessageCallbackFunc = (err: Error | null, chunk: Buffer | null) => void;
export interface ChunkMessageOptions extends SecureMessageChunkManagerOptions {
    tokenId: number;
}
/**
 * @class MessageChunker
 * @param options {Object}
 * @param options.securityHeader  {Object} SecurityHeader
 * @param [options.derivedKeys] {Object} derivedKeys
 * @constructor
 */
export declare class MessageChunker {
    static defaultMaxMessageSize: number;
    static readonly defaultChunkCount: number;
    securityHeader?: AsymmetricAlgorithmSecurityHeader | SymmetricAlgorithmSecurityHeader | null;
    private readonly sequenceNumberGenerator;
    maxMessageSize: number;
    maxChunkCount: number;
    constructor(options?: MessageChunkerOptions);
    dispose(): void;
    /***
     * update security information
     */
    update(options?: MessageChunkerOptions): void;
    chunkSecureMessage(msgType: string, options: ChunkMessageOptions, message: BaseUAObject, messageChunkCallback: MessageCallbackFunc): void;
}
