/**
 * @module node-opcua-server-discovery
 */
import { Message, MessageSecurityMode, Response, SecurityPolicy, ServerSecureChannelLayer } from "node-opcua-secure-channel";
import { OPCUABaseServer, OPCUABaseServerOptions } from "node-opcua-server";
import { MdnsDiscoveryConfiguration, RegisteredServer } from "node-opcua-service-discovery";
import { ApplicationDescription } from "node-opcua-service-endpoints";
import { ErrorCallback } from "node-opcua-status-code";
export interface OPCUADiscoveryServerOptions extends OPCUABaseServerOptions {
    certificateFile?: string;
    port?: number;
    alternateHostname?: string[];
    securityPolicies?: SecurityPolicy[];
    securityModes?: MessageSecurityMode[];
    hostname?: string;
}
export declare class OPCUADiscoveryServer extends OPCUABaseServer {
    private mDnsResponder?;
    private readonly registeredServers;
    private bonjourHolder;
    private _delayInit?;
    constructor(options: OPCUADiscoveryServerOptions);
    start(): Promise<void>;
    start(callback: ErrorCallback): void;
    shutdown(): Promise<void>;
    shutdown(callback: ErrorCallback): void;
    /**
     * returns the number of registered servers
     */
    get registeredServerCount(): number;
    getServers(channel: ServerSecureChannelLayer): ApplicationDescription[];
    protected _on_RegisterServer2Request(message: Message, channel: ServerSecureChannelLayer): void;
    protected _on_RegisterServerRequest(message: Message, channel: ServerSecureChannelLayer): void;
    protected _on_FindServersOnNetworkRequest(message: Message, channel: ServerSecureChannelLayer): void;
    protected __internalRegisterServerWithCallback(RegisterServerXResponse: any, rawServer: RegisteredServer, discoveryConfigurations: MdnsDiscoveryConfiguration[] | undefined, callback: (err: Error | null, response?: Response) => void): Promise<void>;
    protected __internalRegisterServer(RegisterServerXResponse: any, rawServer: RegisteredServer, discoveryConfigurations?: MdnsDiscoveryConfiguration[]): Promise<Response>;
}
